/*
  Maxwell's Lemur
  Copyright (c) 1997 Daniel Risacher <magnus@alum.mit.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <string.h>
#include "core.h"

static void bpage_free_raw(buffer_page *bp) ;

buffer_page *bpage_concat(buffer_page *page1, buffer_page* page2)
{
  buffer_page *temp;
  if (NULL == page1) return page2;
  temp = page1;
  while (temp) {
    page1 = temp;
    temp = page1->next;
  }
  page1->next = page2;
  return page1;
}


buffer_page *find_bpage_by_name(buffer* b, gchar *name)
{
  buffer_page *bp;
  
  if ((bp = b->pages) == NULL)
    return NULL;
  
  do {
    if (!strcmp(name, bp->name))
      return bp;
    bp = bp->next;
  } while (bp != b->pages);
  
  return NULL;
}


/* make a new buffer_page and link it into a buffer b */

buffer_page* bpage_new(buffer* b, gchar *name)
{
  buffer_page *bp;
  bp = (buffer_page *) g_new(buffer_page, 1);
  g_assert(b);
  bp->next = NULL;
  b->pages = bpage_concat(b->pages, bp);
  
  /* FIXME: detect duplicate page names */
  bp->name = g_strdup(name);
  bp->max_row = 0;
  bp->max_col = 0;
  bp->standard_width = 70;
  bp->standard_height = 22;
  bp->nent_rows = 0;
  bp->nent_cols = 0;
  bp->rows = NULL;
  bp->cols = NULL;
  return bp;
}

/*X
  buffer_page *bpage_free(buffer *b, buffer_page *bp) 
  remove bp from page_list and free it's memory 
  X*/
buffer_page *bpage_free(buffer_page *bp, buffer *b) 
{
  buffer_page *pbp, *next;
  next = bp->next;
  if (next == bp)
    next = NULL;
  
  for (pbp = b->pages;
       pbp->next != bp && pbp->next != pbp;
       pbp = pbp->next);
  pbp->next = next;

  if (b->pages == bp)
    b->pages = b->pages->next;

  bpage_free_raw(bp);

  return next;
}

void free_bpage_list(buffer_page *bp, buffer *b) 
{
  while (bpage_free(bp, b));
}

/* free all the sub-parts of bp, then free bp */
static void bpage_free_raw(buffer_page *bp) 
{
  gint i, j;
  for (i = 0; i<bp->nent_rows; i++) {
    row_desc *r;
    r = &bp->rows[i];
    if (r->default_features) /* FIXME */
      g_free(r->default_features);
    for (j=0; j < r->nent_cells; j++) {
      cell *c;
      c = &r->cells[j];
      g_free(c->formula);
      g_free(c->features); /* FIXME */
    }
  }
  g_free(bp->rows);
  for (i=0; i<bp->nent_cols; i++) {
    col_desc *c;
    c = &bp->cols[i];
    if (c->default_features)
      g_free(c->default_features);
  }
  g_free(bp->cols);
  g_free(bp);
}


gint cell_width(buffer_page * bp, gint col)
{
  if (col > bp->nent_cols) return bp->standard_width;
  return bp->cols[col].width;
}

gint cell_height(buffer_page * bp, gint row)
{
  if (row > bp->nent_rows) return bp->standard_height;
  return bp->rows[row].height;
}

void set_width(buffer_page *bp, gint col, gint width)
{
  gint i;

  if (0 == bp->nent_cols) {
    bp->cols = g_new(col_desc, col+1);
    g_assert(bp->cols);
    bp->nent_cols = col+1;
    for (i=0; i<col; i++) {
      bp->cols[i].width = bp->standard_width;
      bp->cols[i].default_features = NULL;
    }
  } else if (col > bp->nent_cols) {
    bp->cols = g_realloc(bp->cols, (col+1)* sizeof(col_desc));
    g_assert(bp->cols);
    for (i=bp->nent_cols+1; i<col; i++) {
      bp->cols[i].width = bp->standard_width;
      bp->cols[i].default_features = NULL;
    }
    bp->nent_cols = col+1;
  }
  bp->cols[col].width = width;
}

void set_height(buffer_page *bp, gint row, gint height)
{
  gint i;

  if (0 == bp->nent_rows) {
    bp->rows = g_new(row_desc, row+1);
    g_assert(bp->rows);
    bp->nent_rows = row+1;
    for (i=0; i<row; i++) {
      bp->rows[i].height = bp->standard_height;
      bp->rows[i].default_features = NULL;
      bp->rows[i].nent_cells = 0;
      bp->rows[i].cells = NULL;
    }
  } else if (row > bp->nent_rows) {
    bp->rows = g_realloc(bp->rows, (row+1)* sizeof(row_desc));
    g_assert(bp->rows);
    for (i=bp->nent_rows+1; i<row; i++) {
      bp->rows[i].height = bp->standard_height;
      bp->rows[i].default_features = NULL;
      bp->rows[i].nent_cells = 0;
      bp->rows[i].cells = NULL;
    }
    bp->nent_rows = row+1;
  }
  bp->rows[row].height = height;
}
    

